@@ -3,7 +3,7 @@
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
  * Distribution prohibited unless authorized in writing. See file COPYING.
- * $Id: zxidmeta.c,v 1.11 2006/09/16 20:00:36 sampo Exp $
+ * $Id: zxidmeta.c,v 1.16 2006/10/15 00:27:26 sampo Exp $
  *
  * 12.8.2006, created --Sampo
  *
@@ -38,19 +38,16 @@
 #include "saml2.h"
 #include "zxid.h"
 #include "zxidconf.h"
-#include "c/saml2-const.h"
-#include "c/saml2-ns.h"
-#include "c/saml2-data.h"
-#include "c/saml2md-const.h"
-#include "c/saml2md-ns.h"
-#include "c/saml2md-data.h"
+#include "c/zx-const.h"
+#include "c/zx-ns.h"
+#include "c/zx-data.h"
 
-#define PEM_CERT_START_MAKER  "-----BEGIN CERTIFICATE-----"
-#define PEM_CERT_END_MAKER    "-----END CERTIFICATE-----"
-#define PEM_RSA_PRIVATE_KEY_START_MAKER  "-----BEGIN RSA PRIVATE KEY-----"
-#define PEM_RSA_PRIVATE_KEY_END_MAKER    "-----END RSA PRIVATE KEY-----"
-#define PEM_DSA_PRIVATE_KEY_START_MAKER  "-----BEGIN DSA PRIVATE KEY-----"
-#define PEM_DSA_PRIVATE_KEY_END_MAKER    "-----END DSA PRIVATE KEY-----"
+#define PEM_CERT_START  "-----BEGIN CERTIFICATE-----"
+#define PEM_CERT_END    "-----END CERTIFICATE-----"
+#define PEM_RSA_PRIV_KEY_START  "-----BEGIN RSA PRIVATE KEY-----"
+#define PEM_RSA_PRIV_KEY_END    "-----END RSA PRIVATE KEY-----"
+#define PEM_DSA_PRIV_KEY_START  "-----BEGIN DSA PRIVATE KEY-----"
+#define PEM_DSA_PRIV_KEY_END    "-----END DSA PRIVATE KEY-----"
 
 /* ============== CoT and Metadata of Others ============== */
 
@@ -122,6 +119,12 @@ struct zxid_entity* zxid_get_meta(struct zxid_conf* cf, char* url)
   rc.lim = rc.p;
   rc.p[1] = 0;
   rc.p = md_buf;
+  if (rc.lim - rc.p < 300) {
+    ERR("Metadata response too short (%d chars, min 300 required). url(%s) CURLcode(%d) CURLerr(%s) buf(%.*s)", rc.lim-md_buf, url, res, curl_easy_strerror(res), rc.lim-md_buf, md_buf);
+    ZX_FREE(cf->ctx, md_buf);
+    return 0;
+  }
+
   ent = zxid_parse_meta(cf, &rc.p, rc.lim);
   if (!ent) {
     ERR("Failed to parse metadata response url(%s) CURLcode(%d) CURLerr(%s) buf(%.*s)", url, res, curl_easy_strerror(res), rc.lim-md_buf, md_buf);
@@ -135,20 +138,46 @@ struct zxid_entity* zxid_get_meta(struct zxid_conf* cf, char* url)
 #endif
 }
 
-struct zxid_entity* zxid_get_meta_ss(struct zxid_conf* cf, struct zx_str_s* url)
+struct zxid_entity* zxid_get_meta_ss(struct zxid_conf* cf, struct zx_str* url)
 {
   return zxid_get_meta(cf, zx_str_to_c(cf->ctx, url));
 }
 
+static void zxid_process_keys(struct zxid_conf* cf, struct zxid_entity* ent,
+			      struct zx_md_KeyDescriptor_s* kd)
+{
+  char* pp;
+  char* p;
+  char* e;
+  X509* x = 0;  /* Forces d2i_X509() to alloc the memory. */
+
+  for (; kd; kd = (struct zx_md_KeyDescriptor_s*)kd->gg.g.n) {
+    p = kd->KeyInfo->X509Data->X509Certificate->content->s;
+    e = p + kd->KeyInfo->X509Data->X509Certificate->content->len;
+    pp = ZX_ALLOC(cf->ctx, SIMPLE_BASE64_PESSIMISTIC_DECODE_LEN(e-p));
+    e = unbase64_raw(p, e, pp, zx_std_index_64);
+    if (!d2i_X509(&x, (const unsigned char**)&pp, e-pp) || !x) {
+      ERR("DER decoding of X509 certificate failed. use(%.*s)", kd->use->len, kd->use->s);
+      return;
+    }
+    if (!memcmp("signing", kd->use->s, kd->use->len)) {
+      ent->sign_cert = x;
+    } else if (!memcmp("encryption", kd->use->s, kd->use->len)) {
+      ent->enc_cert = x;
+    } else {
+      ERR("Unknown key use(%.*s)", kd->use->len, kd->use->s);
+    }
+  }
+}
+
 struct zxid_entity* zxid_parse_meta(struct zxid_conf* cf, char** md, char* lim)
 {
   struct zxid_entity* ent;
-  struct zxmd_md_EntityDescriptor_s* ed;
-  struct zxmd_root_s* r;
-  cf->ctx->ns_tab = zxmd_ns_tab;
-  cf->ctx->base = cf->ctx->p = *md;
-  cf->ctx->lim = lim;
-  r = zxmd_DEC_root(cf->ctx);
+  struct zx_md_EntityDescriptor_s* ed;
+  struct zx_root_s* r;
+
+  zx_prepare_dec_ctx(cf->ctx, zx_ns_tab, *md, lim);
+  r = zx_DEC_root(cf->ctx, 0, 5);
   *md = cf->ctx->p;
   if (!r)
     return 0;
@@ -168,17 +197,25 @@ struct zxid_entity* zxid_parse_meta(struct zxid_conf* cf, char** md, char* lim)
     goto bad_md;
   ent->eid_len = ed->entityID->len;
   ent->eid = ed->entityID->s;
+  ent->sha1_name = ZX_ALLOC(cf->ctx, 28+1);
+  sha1_safe_base64(ent->sha1_name, ent->eid_len, ent->eid);
+
+  if (ed->IDPSSODescriptor)
+    zxid_process_keys(cf, ent, ed->IDPSSODescriptor->KeyDescriptor);
+  if (ed->SPSSODescriptor)
+    zxid_process_keys(cf, ent, ed->SPSSODescriptor->KeyDescriptor);
+
   return ent;
   
  bad_md:
   ERR("Bad metadata. EntityDescriptor could not be found or was corrupt. MD(%.*s) %d chars parsed.", lim-cf->ctx->base, cf->ctx->base, *md - cf->ctx->base);
-  zxmd_FREE_root(cf->ctx, r, 0);
+  zx_FREE_root(cf->ctx, r, 0);
   return 0;
 }
 
 int zxid_write_ent_to_cache(struct zxid_conf* cf, struct zxid_entity* ent)
 {
-  struct zx_str_s* ss;
+  struct zx_str* ss;
   int fd = open_fd_from_path(cf->path, ZXID_COT_DIR, ent->sha1_name,
 			     O_CREAT | O_WRONLY | O_TRUNC, 0666);
   if (fd == -1) {
@@ -187,7 +224,7 @@ int zxid_write_ent_to_cache(struct zxid_conf* cf, struct zxid_entity* ent)
     return 0;
   }
   
-  ss = zxmd_EASY_ENC_SO_md_EntityDescriptor(cf->ctx, ent->ed);
+  ss = zx_EASY_ENC_SO_md_EntityDescriptor(cf->ctx, ent->ed);
   if (!ss)
     return 0;
   write_all_fd(fd, ss->s, ss->len);
@@ -226,15 +263,13 @@ struct zxid_entity* zxid_get_ent_from_file(struct zxid_conf* cf, char* sha1_name
     ZX_FREE(cf->ctx, md_buf);
     return 0;
   }
-  ent->sha1_name = ZX_ALLOC(cf->ctx, strlen(sha1_name)+1);
-  strcpy(ent->sha1_name, sha1_name);
   ent->n = cf->cot;
   cf->cot = ent;
   D("GOT sha1_name(%s)", sha1_name);
   return ent;
 }
 
-struct zxid_entity* zxid_get_ent_from_cache(struct zxid_conf* cf, struct zx_str_s* eid)
+struct zxid_entity* zxid_get_ent_from_cache(struct zxid_conf* cf, struct zx_str* eid)
 {
   struct zxid_entity* ent;
   char sha1_name[28+1];
@@ -245,7 +280,7 @@ struct zxid_entity* zxid_get_ent_from_cache(struct zxid_conf* cf, struct zx_str_
   return zxid_get_ent_from_file(cf, sha1_name);
 }
 
-struct zxid_entity* zxid_get_ent_ss(struct zxid_conf* cf, struct zx_str_s* eid)
+struct zxid_entity* zxid_get_ent_ss(struct zxid_conf* cf, struct zx_str* eid)
 {
   struct zxid_entity* ent;
   
@@ -259,8 +294,6 @@ struct zxid_entity* zxid_get_ent_ss(struct zxid_conf* cf, struct zx_str_s* eid)
   if (cf->md_fetch) {
     ent = zxid_get_meta_ss(cf, eid);
     if (ent) {
-      ent->sha1_name = ZX_ALLOC(cf->ctx, 28+1);
-      sha1_safe_base64(ent->sha1_name, ent->eid_len, ent->eid);
       ent->n = cf->cot;
       cf->cot = ent;
       
@@ -281,7 +314,7 @@ struct zxid_entity* zxid_get_ent_ss(struct zxid_conf* cf, struct zx_str_s* eid)
 
 struct zxid_entity* zxid_get_ent(struct zxid_conf* cf, char* eid)
 {
-  struct zx_str_s ss;
+  struct zx_str ss;
   if (!eid)
     return 0;
   ss.s = eid;
@@ -336,7 +369,8 @@ struct zxid_entity* zxid_load_cot_cache(struct zxid_conf* cf)
   }
   
   while (de = readdir(dir))
-    zxid_get_ent_by_sha1_name(cf, de->d_name);
+    if (de->d_name[0] != '.')
+      zxid_get_ent_by_sha1_name(cf, de->d_name);
   
   closedir(dir);
   return cf->cot;
@@ -344,17 +378,17 @@ struct zxid_entity* zxid_load_cot_cache(struct zxid_conf* cf)
 
 /* ============== Our Metadata ============== */
 
-struct zxmd_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X509* x)
+struct zx_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X509* x)
 {
   int len;
   char* dd;
   char* d;
   char* pp;
   char* p;
-  struct zxmd_md_KeyDescriptor_s* kd = zxmd_NEW_md_KeyDescriptor(cf->ctx);
+  struct zx_md_KeyDescriptor_s* kd = zx_NEW_md_KeyDescriptor(cf->ctx);
   kd->use = zx_ref_str(cf->ctx, use);
-  kd->KeyInfo = zxmd_NEW_ds_KeyInfo(cf->ctx);
-  kd->KeyInfo->X509Data = zxmd_NEW_ds_X509Data(cf->ctx);
+  kd->KeyInfo = zx_NEW_ds_KeyInfo(cf->ctx);
+  kd->KeyInfo->X509Data = zx_NEW_ds_X509Data(cf->ctx);
 
 #ifdef USE_OPENSSL
   /* Build PEM encoding */
@@ -366,14 +400,13 @@ struct zxmd_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X
     dd = d = ZX_ALLOC(cf->ctx, len);
     i2d_X509(x, (unsigned char**)&d);
     pp = p = ZX_ALLOC(cf->ctx, (len+4) * 4 / 3 + (len/64) + 6
-		      + sizeof(PEM_CERT_START_MAKER)
-		      + sizeof(PEM_CERT_END_MAKER));    
-    memcpy(p, PEM_CERT_START_MAKER, sizeof(PEM_CERT_START_MAKER)-1);
-    p += sizeof(PEM_CERT_START_MAKER)-1;
+		      + sizeof(PEM_CERT_START) + sizeof(PEM_CERT_END));    
+    memcpy(p, PEM_CERT_START, sizeof(PEM_CERT_START)-1);
+    p += sizeof(PEM_CERT_START)-1;
     *p++ = '\n';
     p = base64_fancy_raw(dd, len, p, std_basis_64, 64, 1, "\n", '=');
-    memcpy(p, PEM_CERT_END_MAKER, sizeof(PEM_CERT_END_MAKER)-1);
-    p += sizeof(PEM_CERT_END_MAKER)-1;
+    memcpy(p, PEM_CERT_END, sizeof(PEM_CERT_END)-1);
+    p += sizeof(PEM_CERT_END)-1;
     /**p++ = '\n';*/
     *p = 0;
 
@@ -385,10 +418,10 @@ struct zxmd_md_KeyDescriptor_s* zxid_key_desc(struct zxid_conf* cf, char* use, X
   return kd;
 }
 
-struct zxmd_md_SingleLogoutService_s* zxid_slo_desc(struct zxid_conf* cf,
+struct zx_md_SingleLogoutService_s* zxid_slo_desc(struct zxid_conf* cf,
 						    char* binding, char* loc, char* resp_loc)
 {
-  struct zxmd_md_SingleLogoutService_s* d = zxmd_NEW_md_SingleLogoutService(cf->ctx);
+  struct zx_md_SingleLogoutService_s* d = zx_NEW_md_SingleLogoutService(cf->ctx);
   d->Binding = zx_ref_str(cf->ctx, binding);
   d->Location = zx_strf(cf->ctx, "%s%s", cf->url, loc);
   if (resp_loc)
@@ -396,10 +429,10 @@ struct zxmd_md_SingleLogoutService_s* zxid_slo_desc(struct zxid_conf* cf,
   return d;
 }
 
-struct zxmd_md_ManageNameIDService_s* zxid_nireg_desc(struct zxid_conf* cf,
+struct zx_md_ManageNameIDService_s* zxid_nireg_desc(struct zxid_conf* cf,
 						      char* binding, char* loc, char* resp_loc)
 {
-  struct zxmd_md_ManageNameIDService_s* d = zxmd_NEW_md_ManageNameIDService(cf->ctx);
+  struct zx_md_ManageNameIDService_s* d = zx_NEW_md_ManageNameIDService(cf->ctx);
   d->Binding = zx_ref_str(cf->ctx, binding);
   d->Location = zx_strf(cf->ctx, "%s%s", cf->url, loc);
   if (resp_loc)
@@ -407,64 +440,64 @@ struct zxmd_md_ManageNameIDService_s* zxid_nireg_desc(struct zxid_conf* cf,
   return d;
 }
 
-struct zxmd_md_AssertionConsumerService_s* zxid_ac_desc(struct zxid_conf* cf,
+struct zx_md_AssertionConsumerService_s* zxid_ac_desc(struct zxid_conf* cf,
 							char* binding, char* loc, char* ix)
 {
-  struct zxmd_md_AssertionConsumerService_s* d = zxmd_NEW_md_AssertionConsumerService(cf->ctx);
+  struct zx_md_AssertionConsumerService_s* d = zx_NEW_md_AssertionConsumerService(cf->ctx);
   d->Binding = zx_ref_str(cf->ctx, binding);
   d->Location = zx_strf(cf->ctx, "%s%s", cf->url, loc);
   d->index = zx_ref_str(cf->ctx, ix);
   return d;
 }
 
-struct zxmd_md_SPSSODescriptor_s* zxid_sp_sso_desc(struct zxid_conf* cf)
+struct zx_md_SPSSODescriptor_s* zxid_sp_sso_desc(struct zxid_conf* cf)
 {
-  struct zxmd_md_SPSSODescriptor_s* sp_ssod = zxmd_NEW_md_SPSSODescriptor(cf->ctx);
+  struct zx_md_SPSSODescriptor_s* sp_ssod = zx_NEW_md_SPSSODescriptor(cf->ctx);
   sp_ssod->AuthnRequestsSigned        = zx_ref_str(cf->ctx, cf->authn_req_sign);
   sp_ssod->WantAssertionsSigned       = zx_ref_str(cf->ctx, cf->want_sso_a7n_signed);
   sp_ssod->errorURL                   = zx_strf(cf->ctx, "%s?o=E", cf->url);
   sp_ssod->protocolSupportEnumeration = zx_ref_str(cf->ctx, SAML2_PROTO);
 
-  zxmd_md_SPSSODescriptor_PUSH_KeyDescriptor(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_KeyDescriptor(sp_ssod,
 					zxid_key_desc(cf, "encryption", cf->enc_cert));
-  zxmd_md_SPSSODescriptor_PUSH_KeyDescriptor(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_KeyDescriptor(sp_ssod,
 					zxid_key_desc(cf, "signing", cf->sign_cert));
 
-  zxmd_md_SPSSODescriptor_PUSH_SingleLogoutService(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_SingleLogoutService(sp_ssod,
      zxid_slo_desc(cf, SAML2_REDIR, "?o=K", "?o=K"));
-  zxmd_md_SPSSODescriptor_PUSH_SingleLogoutService(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_SingleLogoutService(sp_ssod,
      zxid_slo_desc(cf, SAML2_SOAP, "?o=S", 0));
 
-  zxmd_md_SPSSODescriptor_PUSH_ManageNameIDService(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_ManageNameIDService(sp_ssod,
      zxid_nireg_desc(cf, SAML2_REDIR, "?o=I", "?o=I"));
-  zxmd_md_SPSSODescriptor_PUSH_ManageNameIDService(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_ManageNameIDService(sp_ssod,
      zxid_nireg_desc(cf, SAML2_SOAP, "?o=S", 0));
 
-  zxmd_md_SPSSODescriptor_PUSH_NameIDFormat(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_NameIDFormat(sp_ssod,
      zx_ref_simple_elem(cf->ctx, SAML2_PERSISTENT_NID_FMT));
-  zxmd_md_SPSSODescriptor_PUSH_NameIDFormat(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_NameIDFormat(sp_ssod,
      zx_ref_simple_elem(cf->ctx, SAML2_TRANSIENT_NID_FMT));
 
   /* N.B. The index values should not be changed. They are used in
    * AuthnReq to choose profile using AssertionConsumerServiceIndex */
-  zxmd_md_SPSSODescriptor_PUSH_AssertionConsumerService(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_AssertionConsumerService(sp_ssod,
      zxid_ac_desc(cf, SAML2_PAOS, "?o=A", "4"));
-  zxmd_md_SPSSODescriptor_PUSH_AssertionConsumerService(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_AssertionConsumerService(sp_ssod,
      zxid_ac_desc(cf, SAML2_SOAP, "?o=S", "3"));
-  zxmd_md_SPSSODescriptor_PUSH_AssertionConsumerService(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_AssertionConsumerService(sp_ssod,
      zxid_ac_desc(cf, SAML2_POST, "?o=P", "2"));
-  zxmd_md_SPSSODescriptor_PUSH_AssertionConsumerService(sp_ssod,
+  zx_md_SPSSODescriptor_PUSH_AssertionConsumerService(sp_ssod,
      zxid_ac_desc(cf, SAML2_ART, "", "1"));
   
   return sp_ssod;
 }
 
-struct zx_str_s* zxid_my_entity_id(struct zxid_conf* cf)
+struct zx_str* zxid_my_entity_id(struct zxid_conf* cf)
 {
   return zx_strf(cf->ctx, "%s?o=B", cf->url);
 }
 
-struct zx_str_s* zxid_my_cdc_url(struct zxid_conf* cf)
+struct zx_str* zxid_my_cdc_url(struct zxid_conf* cf)
 {
   return zx_strf(cf->ctx, "%s?o=C", cf->cdc_url);
 }
@@ -478,20 +511,20 @@ struct zx_sa_Issuer_s* zxid_my_issuer(struct zxid_conf* cf)
   return is;
 }
 
-struct zx_str_s* zxid_sp_meta(struct zxid_conf* cf, struct zxid_cgi* cgi)
+struct zx_str* zxid_sp_meta(struct zxid_conf* cf, struct zxid_cgi* cgi)
 {
-  struct zxmd_md_EntityDescriptor_s* ed;
+  struct zx_md_EntityDescriptor_s* ed;
   
-  ed = zxmd_NEW_md_EntityDescriptor(cf->ctx);
+  ed = zx_NEW_md_EntityDescriptor(cf->ctx);
   ed->entityID = zxid_my_entity_id(cf);
   ed->SPSSODescriptor = zxid_sp_sso_desc(cf);
   
-  return zxmd_EASY_ENC_SO_md_EntityDescriptor(cf->ctx, ed);
+  return zx_EASY_ENC_SO_md_EntityDescriptor(cf->ctx, ed);
 }
 
 int zxid_send_sp_meta(struct zxid_conf* cf, struct zxid_cgi* cgi)
 {
-  struct zx_str_s* ss = zxid_sp_meta(cf, cgi);
+  struct zx_str* ss = zxid_sp_meta(cf, cgi);
   if (!ss)
     return 0;
   write_all_fd(1, ss->s, ss->len);
