@@ -3,7 +3,7 @@
  * This is confidential unpublished proprietary source code of the author.
  * NO WARRANTY, not even implied warranties. Contains trade secrets.
  * Distribution prohibited unless authorized in writing. See file COPYING.
- * $Id: zxlib.c,v 1.7 2006/09/16 20:00:36 sampo Exp $
+ * $Id: zxlib.c,v 1.16 2006/10/15 00:27:26 sampo Exp $
  *
  * 28.5.2006, created --Sampo
  * 8.8.2006,  moved lookup functions to generated code --Sampo
@@ -20,9 +20,9 @@
 
 #include "errmac.h"
 #include "zx.h"
-#include "c/saml2-data.h"  /* ALso generig zx simple_elem, etc. */
+#include "c/zx-data.h"  /* ALso generig zx simple_elem, etc. */
 
-char* zx_alloc(struct zx_ctx* c, int size)
+void* zx_alloc(struct zx_ctx* c, int size)
 {
   char* p;
   p = malloc(size);
@@ -30,21 +30,21 @@ char* zx_alloc(struct zx_ctx* c, int size)
   return p;
 }
 
-char* zx_zalloc(struct zx_ctx* c, int size)
+void* zx_zalloc(struct zx_ctx* c, int size)
 {
   char* p = zx_alloc(c, size);
   memset(p, 0, size);
   return p;
 }
 
-char* zx_free(struct zx_ctx* c, void* p)
+void* zx_free(struct zx_ctx* c, void* p)
 {
   if (p)
     free(p);
   return 0;
 }
 
-char* zx_str_to_c(struct zx_ctx* c, struct zx_str_s* ss)
+char* zx_str_to_c(struct zx_ctx* c, struct zx_str* ss)
 {
   char* p = ZX_ALLOC(c, ss->len+1);
   memcpy(p, ss->s, ss->len);
@@ -55,7 +55,7 @@ char* zx_str_to_c(struct zx_ctx* c, struct zx_str_s* ss)
 /* zx_str_conv() helps SWIG typemaps to achieve natural conversion
  * to native length + data representations of scripting languages. */
 
-void zx_str_conv(struct zx_str_s* ss, int* out_len, char** out_s)  /* SWIG typemap friendly */
+void zx_str_conv(struct zx_str* ss, int* out_len, char** out_s)  /* SWIG typemap friendly */
 {
   *out_s = 0;
   *out_len = 0;
@@ -65,31 +65,31 @@ void zx_str_conv(struct zx_str_s* ss, int* out_len, char** out_s)  /* SWIG typem
   *out_len = ss->len;
 }
 
-void zx_str_free(struct zx_ctx* c, struct zx_str_s* ss)
+void zx_str_free(struct zx_ctx* c, struct zx_str* ss)
 {
   if (ss->s)
     ZX_FREE(c, ss->s);
   ZX_FREE(c, ss);
 }
 
-struct zx_str_s* zx_ref_len_str(struct zx_ctx* c, int len, char* s)
+struct zx_str* zx_ref_len_str(struct zx_ctx* c, int len, char* s)
 {
-  struct zx_str_s* ss;
-  ss = (struct zx_str_s*)ZX_ALLOC(c, sizeof(struct zx_str_s));
+  struct zx_str* ss;
+  ss = (struct zx_str*)ZX_ALLOC(c, sizeof(struct zx_str));
   ss->s = s;  /* ref points to underlying data */
   ss->len = len;
   return ss;
 }
 
-struct zx_str_s* zx_ref_str(struct zx_ctx* c, char* s)
+struct zx_str* zx_ref_str(struct zx_ctx* c, char* s)
 {
   return zx_ref_len_str(c, strlen(s), s);
 }
 
-struct zx_str_s* zx_dup_len_str(struct zx_ctx* c, int len, char* s)
+struct zx_str* zx_dup_len_str(struct zx_ctx* c, int len, char* s)
 {
-  struct zx_str_s* ss;
-  ss = (struct zx_str_s*)ZX_ALLOC(c, sizeof(struct zx_str_s));
+  struct zx_str* ss;
+  ss = (struct zx_str*)ZX_ALLOC(c, sizeof(struct zx_str));
   ss->s = ZX_ALLOC(c, len+1);
   memcpy(ss->s, s, len);
   ss->s[len] = 0;
@@ -97,12 +97,12 @@ struct zx_str_s* zx_dup_len_str(struct zx_ctx* c, int len, char* s)
   return ss;
 }
 
-struct zx_str_s* zx_dup_str(struct zx_ctx* c, char* s)
+struct zx_str* zx_dup_str(struct zx_ctx* c, char* s)
 {
   return zx_dup_len_str(c, strlen(s), s);
 }
 
-struct zx_str_s* zx_strf(struct zx_ctx* c, char* f, ...)  /* data is new memory */
+struct zx_str* zx_strf(struct zx_ctx* c, char* f, ...)  /* data is new memory */
 {
   va_list ap;
   int len;
@@ -117,7 +117,12 @@ struct zx_str_s* zx_strf(struct zx_ctx* c, char* f, ...)  /* data is new memory
   return zx_ref_len_str(c, len, s);
 }
 
-struct zx_elem_s* zx_new_simple_elem(struct zx_ctx* c, struct zx_str_s* ss)
+int zx_str_ends_in(struct zx_str* ss, int len, char* suffix)
+{
+  return !memcmp(ss->s + ss->len - len, suffix, len);
+}
+
+struct zx_elem_s* zx_new_simple_elem(struct zx_ctx* c, struct zx_str* ss)
 {
   struct zx_elem_s* el;
   el = ZX_ZALLOC(c, struct zx_elem_s);
@@ -192,7 +197,7 @@ struct zx_ns_s* zx_locate_ns_by_prefix(struct zx_ctx* c, int len, char* prefix)
   return 0;
 }
 
-struct zx_ns_s* zx_locate_ns_by_url(struct zx_ctx* c, int len, char* url)
+static struct zx_ns_s* zx_locate_ns_by_url(struct zx_ctx* c, int len, char* url)
 {
   struct zx_ns_s* ns;
   struct zx_ns_s* alias;
@@ -201,7 +206,7 @@ struct zx_ns_s* zx_locate_ns_by_url(struct zx_ctx* c, int len, char* url)
       return ns;
     for (alias = ns->n; alias; alias = alias->n)
       if (alias->url_len == len && (!len || !memcmp(alias->url, url, len)))
-	return ns;
+	return alias;
   }
 
   /* Trailer element holds unknown namespaces. */
@@ -211,7 +216,7 @@ struct zx_ns_s* zx_locate_ns_by_url(struct zx_ctx* c, int len, char* url)
   return 0;
 }
 
-void zx_xmlns_decl(struct zx_ctx* c, int prefix_len, char* prefix, int url_len, char* url)
+static struct zx_ns_s* zx_xmlns_decl(struct zx_ctx* c, int prefix_len, char* prefix, int url_len, char* url)
 {
   struct zx_ns_s* alias;
   struct zx_ns_s* ns = zx_locate_ns_by_url(c, url_len, url);
@@ -223,8 +228,7 @@ void zx_xmlns_decl(struct zx_ctx* c, int prefix_len, char* prefix, int url_len,
       return;  /* Namespace not known by compiled in schemata. */
     }
   }
-  if (zx_is_ns_prefix(ns, prefix_len, prefix))
-    return;
+  /* Always alloc a new one because we may need to push to stack multiple instances of same. */
   alias = ZX_ZALLOC(c, struct zx_ns_s);
   alias->prefix_len = prefix_len;
   alias->prefix = prefix;
@@ -232,6 +236,69 @@ void zx_xmlns_decl(struct zx_ctx* c, int prefix_len, char* prefix, int url_len,
   alias->url =url;
   alias->n = ns->n;
   ns->n = alias;
+  return alias;
+}
+
+struct zx_ns_s* zx_prefix_seen(struct zx_ctx* c, int len, char* prefix)
+{
+  struct zx_ns_s* ns;
+  for (ns = c->guard_seen_n.seen_n; ns->seen_n; ns = ns->seen_n)
+    if (ns->prefix_len == len && (!len || !memcmp(ns->prefix, prefix, len)))
+      return ns;
+  return 0;
+}
+
+struct zx_ns_s* zx_prefix_seen_whine(struct zx_ctx* c, int len, char* prefix, char* logkey)
+{
+  struct zx_ns_s* ns = zx_prefix_seen(c, len, prefix);
+  if (!ns)
+    D("Undefined namespace prefix(%.*s) at(%s)", len, prefix, logkey);
+  return ns;
+}
+
+/* See if prefix has been seen, and in the same meaning. If not, the allocate
+ * a new node and push or add it to the doubly linked list as well as to the
+ * pop_seen list. Returns 0 if no addition was done (i.e. ns had been seen already). */
+
+static struct zx_ns_s* zx_push_seen(struct zx_ctx* c, int prefix_len, char* prefix, int url_len, char* url, struct zx_ns_s** pop_seen)
+{
+  struct zx_ns_s* old_ns;
+  struct zx_ns_s* ns;
+  old_ns = zx_prefix_seen(c, prefix_len, prefix);
+  if (old_ns) {
+    if (old_ns->url_len == url_len && (!url_len || !memcmp(old_ns->url, url, url_len)))
+      return 0;
+    ns = zx_xmlns_decl(c, prefix_len, prefix, url_len, url);
+    ns->seen = old_ns;            /* Push */
+    ns->seen_n = old_ns->seen_n;  /* Replace old_ns in middle of the list */
+    ns->seen_p = old_ns->seen_p;
+    old_ns->seen_n->seen_p = ns;
+    old_ns->seen_p->seen_n = ns;
+  } else {
+    ns = zx_xmlns_decl(c, prefix_len, prefix, url_len, url);
+    ns->seen_n = c->guard_seen_n.seen_n;  /* Add to beginning of seen_n list. */
+    c->guard_seen_n.seen_n = ns;
+    ns->seen_n->seen_p = ns;
+    ns->seen_p = &c->guard_seen_n;
+  }
+  ns->seen_pop = *pop_seen;
+  *pop_seen = ns;
+  return ns;
+}
+
+void zx_pop_seen(struct zx_ns_s* ns)
+{
+  for (; ns; ns = ns->seen_pop) {
+    if (ns->seen) {
+      ns->seen->seen_n = ns->seen_n;   /* Replace ns with old_ns (ns->seen) in middle of list */
+      ns->seen->seen_p = ns->seen_p;
+      ns->seen_n->seen_p = ns->seen;
+      ns->seen_p->seen_n = ns->seen;
+    } else {
+      ns->seen_n->seen_p = ns->seen_p; /* Remove ns from middle of the seen_n list. */
+      ns->seen_p->seen_n = ns->seen_n;
+    }
+  }
 }
 
 /* When trying to scan an element, an annoying feature of XML namespaces is that the
@@ -239,20 +306,25 @@ void zx_xmlns_decl(struct zx_ctx* c, int prefix_len, char* prefix, int url_len,
  * at the time of scanning the <ns:element part we can't know its namespace. What
  * a lousy design. In order to handle this we need to either backtrack or
  * make a special case forward scan for xmlns attributes (which is redundant with
- * normal attribute scanning). It seems simpler to do the latter, so here goes... */
+ * normal attribute scanning). It seems simpler to do the latter, so here goes...
+ *
+ * The return value represents the list of namespaces that were newly declared
+ * at this level, i.e. pushed to the seen stacks. This list is used to pop the
+ * seen stacks after we are through with the element. */
 
 struct zx_ns_s* zx_scan_xmlns(struct zx_ctx* c)
 {
-  struct zx_ns_s* ns = 0;  /* *** build a list of namespaces declared? */
+  struct zx_ns_s* pop_seen = 0;  /* build a list of namespaces declared here */
   char* prefix;
   char* url;
   char* p = c->p;  /* We need to keep the original c->p so normal attributes can be scanned. */
   char quote;
-  
+  int prefix_len, url_len;
+
   /* The tag name has already been detected. Process attributes until '>' */
   
   for ( ; 1; ++p) {
-    ZX_SKIP_WS_P(c,p,ns);
+    ZX_SKIP_WS_P(c, p, pop_seen);
     if (ONE_OF_2(*p, '>', '/'))
       break;
     if (!memcmp(p, "xmlns", sizeof("xmlns")-1)) {
@@ -262,33 +334,59 @@ struct zx_ns_s* zx_scan_xmlns(struct zx_ctx* c)
 	goto scan_URL;
       case ':':  /* Qualified namespace decl. */
 	prefix = p += 6;
-	ZX_LOOK_FOR_P(c,'=',p,ns);
+	ZX_LOOK_FOR_P(c,'=',p, pop_seen);
       scan_URL:
 	++p;
 	if (!ONE_OF_2(*p, '"', '\''))
-	  return ns;
+	  return pop_seen;
 	quote = *p;
 	url = ++p;
-	ZX_LOOK_FOR_P(c,quote,p,ns);
-	zx_xmlns_decl(c, (url - 2) - prefix, prefix, p - url, url);
-	goto next;	
+	ZX_LOOK_FOR_P(c, quote, p, pop_seen);
+
+	prefix_len = (url - 2) - prefix;
+	url_len = p - url;
+	zx_push_seen(c, prefix_len, prefix, url_len, url, &pop_seen);
+	goto next;
       default:
 	D("Illformed attributes. Bad char(%c)", p[5]);
-	return ns;
+	return pop_seen;
       }
     }
     
-    ZX_LOOK_FOR_P(c,'=',p,ns);
+    /* Skip over any other attributes. */
+    
+    ZX_LOOK_FOR_P(c,'=',p, pop_seen);
     ++p;
     if (!ONE_OF_2(*p, '"', '\''))
-      return ns;
+      return pop_seen;
     quote = *p;
     ++p;
-    ZX_LOOK_FOR_P(c,quote,p,ns);
+    ZX_LOOK_FOR_P(c,quote,p, pop_seen);
   next:
     continue;
   }
-  return ns;
+  return pop_seen;
+}
+
+const struct zx_tok* zx_tok_by_ns(const struct zx_tok* zt, const struct zx_tok* lim,
+				  int len, char* name, struct zx_ns_s* ns)
+{
+  struct zx_ns_s* alias;
+  const struct zx_tok* ztt;
+
+  /* First find token whose name matches. The token table CAN have duplicate entries,
+   * see -D flag to gperf. */
+  for (; zt < lim && (memcmp(zt->name, name, len) || zt->name[len]); ++zt) ;
+  ztt = zt < lim ? zt : 0;  /* Remember in case we fail. */
+  if (!ns)
+    return ztt;
+  
+  /* Now check for namespace match. */
+  for (; zt < lim && (!memcmp(zt->name, name, len) && !zt->name[len]); ++zt)
+    for (alias = zt->ns; alias; alias = alias->n)
+      if (alias == ns)
+	return zt;
+  return ztt;
 }
 
 #if 0
@@ -305,12 +403,93 @@ int zx_init_tok_tab(struct zx_ctx* c, struct zx_tok* tok_tab, struct zx_tok* tok
 }
 #endif
 
+int zx_len_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seen)
+{
+  if (!zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, pop_seen))
+    return 0;
+  return sizeof(" xmlns")-1
+    + (ns->prefix_len ? ns->prefix_len+1 : 0) + 2 + ns->url_len + 1;
+}
+
+char* zx_enc_xmlns_if_not_seen(struct zx_ctx* c, char* p, struct zx_ns_s* ns, struct zx_ns_s** pop_seen)
+{
+  if (!zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, pop_seen))
+    return p;
+  
+  ZX_OUT_MEM(p, " xmlns", sizeof(" xmlns")-1);
+  if (ns->prefix_len) {
+    ZX_OUT_CH(p, ':');
+    ZX_OUT_MEM(p, ns->prefix, ns->prefix_len);
+  }
+  ZX_OUT_CH(p, '=');
+  ZX_OUT_CH(p, '"');
+  ZX_OUT_MEM(p, ns->url, ns->url_len);
+  ZX_OUT_CH(p, '"');
+  return p;
+}
+
+/* For WO encoder the sort of xmlns declarations is not known at compile
+ * time. Thus we first add them to the pop_seen list using insertion
+ * sort (pop_seen is smallest and prefixes grow from there) and
+ * then later render the list using zx_enc_seen(). */
+
+void zx_add_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seen)
+{
+  struct zx_ns_s* pop_seen_dummy=0;
+  struct zx_ns_s* new_ns;
+  int res;
+  new_ns = zx_push_seen(c, ns->prefix_len, ns->prefix, ns->url_len, ns->url, &pop_seen_dummy);
+  if (!new_ns)
+    return;
+  if (!*pop_seen) {
+    *pop_seen = new_ns;
+    return;
+  }
+  if (!new_ns->prefix_len) {       /* Default namespace (empty prefix) sorts first. */
+first:
+    new_ns->seen_pop = *pop_seen;
+    *pop_seen = new_ns;
+    return;
+  }
+
+  ns = *pop_seen;
+  if (ns->prefix_len) {
+    res = memcmp(ns->prefix, new_ns->prefix, MIN(ns->prefix_len, new_ns->prefix_len));
+    if ((res > 0) || !res && ns->prefix_len >= new_ns->prefix_len)
+      goto first;
+  }
+  for (; ns->seen_pop; ns = ns->seen_pop) {
+    res = memcmp(ns->seen_pop->prefix, new_ns->prefix,
+		 MIN(ns->seen_pop->prefix_len, new_ns->prefix_len));
+    if ((res > 0) || !res && ns->seen_pop->prefix_len >= new_ns->prefix_len)
+      break;
+  }
+  new_ns->seen_pop = ns->seen_pop;
+  ns->seen_pop = new_ns;
+}
+
+char* zx_enc_seen(char* p, struct zx_ns_s* ns)
+{
+  for (; ns; ns = ns->seen_pop) {
+    ZX_OUT_MEM(p, " xmlns", sizeof(" xmlns")-1);
+    if (ns->prefix_len) {
+      ZX_OUT_CH(p, ':');
+      ZX_OUT_MEM(p, ns->prefix, ns->prefix_len);
+    }
+    ZX_OUT_CH(p, '=');
+    ZX_OUT_CH(p, '"');
+    ZX_OUT_MEM(p, ns->url, ns->url_len);
+    ZX_OUT_CH(p, '"');
+  }
+  return p;
+}
+
 /* Render the unknown attributes list. CSE for almost all tags. */
 
-int zx_len_common(struct zx_elem_s* x)
+int zx_len_so_common(struct zx_ctx* c, struct zx_elem_s* x)
 {
   int len = 0;
-  struct zx_str_s* ss;
+  struct zx_str* ss;
   struct zx_any_attr_s* aa;
   struct zx_any_elem_s* ae;
   
@@ -321,15 +500,37 @@ int zx_len_common(struct zx_elem_s* x)
   }
 
   for (ae = x->any_elem; ae; ae = (struct zx_any_elem_s*)ae->gg.g.n)    /* unknown elements */
-    len += zx_LEN_simple_elem(&ae->gg, ae->name_len);
+    len += zx_LEN_SO_simple_elem(c, &ae->gg, ae->name_len, 0);          /* *** figure namespaces */
   
-  for (ss = x->content; ss; ss = (struct zx_str_s*)ss->g.n)             /* content */
+  for (ss = x->content; ss; ss = (struct zx_str*)ss->g.n)             /* content */
     len += ss->len;
   
   return len;
 }
 
-char* zx_enc_so_unknown_attrs(char* p, struct zx_any_attr_s* aa)
+int zx_len_wo_common(struct zx_ctx* c, struct zx_elem_s* x)
+{
+  int len = 0;
+  struct zx_str* ss;
+  struct zx_any_attr_s* aa;
+  struct zx_any_elem_s* ae;
+  
+  for (aa = x->any_attr; aa; aa = (struct zx_any_attr_s*)aa->ss.g.n) {  /* unknown attributes */
+    if (aa->ss.g.ns && aa->ss.g.ns->prefix_len)
+      len += aa->ss.g.ns->prefix_len + 1;
+    len += 1 + aa->name_len + 1 + 1 + aa->ss.len + 1;  /* attr="val" */
+  }
+
+  for (ae = x->any_elem; ae; ae = (struct zx_any_elem_s*)ae->gg.g.n)    /* unknown elements */
+    len += zx_LEN_WO_simple_elem(c, &ae->gg, ae->name_len);
+  
+  for (ss = x->content; ss; ss = (struct zx_str*)ss->g.n)             /* content */
+    len += ss->len;
+  
+  return len;
+}
+
+char* zx_enc_unknown_attrs(char* p, struct zx_any_attr_s* aa)
 {
   for (; aa; aa = (struct zx_any_attr_s*)aa->ss.g.n) {  /* unknown attributes */
     ZX_OUT_CH(p, ' ');
@@ -344,24 +545,25 @@ char* zx_enc_so_unknown_attrs(char* p, struct zx_any_attr_s* aa)
     ZX_OUT_MEM(p, aa->ss.s, aa->ss.len);
     ZX_OUT_CH(p, '"');
   }
+  ZX_OUT_CH(p, '>');
   return p;
 }
 
-char* zx_enc_so_unknown_elems_and_content(char* p, struct zx_elem_s* x)
+char* zx_enc_so_unknown_elems_and_content(struct zx_ctx* c, char* p, struct zx_elem_s* x)
 {
-  struct zx_str_s* ss;
+  struct zx_str* ss;
   struct zx_any_elem_s* ae;
   
   for (ae = x->any_elem; ae; ae = (struct zx_any_elem_s*)ae->gg.g.n)    /* unknown elements */
-    p = zx_ENC_SO_simple_elem(&ae->gg, p, ae->name, ae->name_len);
+    p = zx_ENC_SO_simple_elem(c, &ae->gg, p, ae->name, ae->name_len, ae->gg.g.ns);
   
-  for (ss = x->content; ss; ss = (struct zx_str_s*)ss->g.n)      /* content */
+  for (ss = x->content; ss; ss = (struct zx_str*)ss->g.n)             /* content */
     ZX_OUT_MEM(p, ss->s, ss->len);
   
   return p;
 }
 
-struct zx_str_s* zx_easy_enc_common(struct zx_ctx* c, char* p, char* buf, int len)
+struct zx_str* zx_easy_enc_common(struct zx_ctx* c, char* p, char* buf, int len)
 {
   if (p != buf+len) {
     ERR("Encoded length(%d) does not match computed length(%d). ED(%.*s)", p-buf, len, p-buf, buf);
@@ -371,41 +573,58 @@ struct zx_str_s* zx_easy_enc_common(struct zx_ctx* c, char* p, char* buf, int le
   return zx_ref_len_str(c, len, buf);
 }
 
-int zx_attr_len(struct zx_str_s* attr, int name_size)
+int zx_attr_so_len(struct zx_str* attr, int name_size)
 {
   int len = 0;
   /* In legal XML there should really be just one attribute, but we acommodate multioccurances. */
-  for (; attr; attr = (struct zx_str_s*)attr->g.n) {
-    if (attr->g.ns && attr->g.ns->prefix_len)
-      len += attr->g.ns->prefix_len + 1;
-    len += 1 + name_size - 1 + 1 + 1 + attr->len + 1;
-  }
+  for (; attr; attr = (struct zx_str*)attr->g.n)
+    len += 1 + name_size + 1 + 1 + attr->len + 1;
   return len;
 }
 
-char* zx_attr_enc(char* p, struct zx_str_s* attr, char* name, int name_len)
+char* zx_attr_so_enc(char* p, struct zx_str* attr, char* name, int name_len)
 {
   /* In legal XML there should really be just one attribute, but we acommodate multioccurances. */
-  for (; attr; attr = (struct zx_str_s*)attr->g.n) {
-    ZX_OUT_CH(p, ' ');
-    if (attr->g.ns && attr->g.ns->prefix_len) {
-      ZX_OUT_MEM(p, attr->g.ns->prefix, attr->g.ns->prefix_len);
-      ZX_OUT_CH(p, ':');
-    }
-    
+  for (; attr; attr = (struct zx_str*)attr->g.n) {
     ZX_OUT_MEM(p, name, name_len);
-    ZX_OUT_CH(p, '=');
-    ZX_OUT_CH(p, '"');
     ZX_OUT_MEM(p, attr->s, attr->len);
     ZX_OUT_CH(p, '"');
   }
   return p;
 }
 
-void zx_dup_attr(struct zx_ctx* c, struct zx_str_s* attr)
+int zx_attr_wo_len(struct zx_str* attr, int name_size)
+{
+  int len = 0;
+  /* In legal XML there should really be just one attribute, but we acommodate multioccurances. */
+  for (; attr; attr = (struct zx_str*)attr->g.n) {
+    if (attr->g.ns && attr->g.ns->prefix_len)
+      len += attr->g.ns->prefix_len + 1;
+    len += 1 + name_size + 1 + 1 + attr->len + 1;
+  }
+  return len;
+}
+
+char* zx_attr_wo_enc(char* p, struct zx_str* attr, char* name, int name_len)
+{
+  /* In legal XML there should really be just one attribute, but we acommodate multioccurances. */
+  for (; attr; attr = (struct zx_str*)attr->g.n) {
+    ZX_OUT_CH(p, ' ');
+    if (attr->g.ns && attr->g.ns->prefix_len) {
+      ZX_OUT_MEM(p, attr->g.ns->prefix, attr->g.ns->prefix_len);
+      ZX_OUT_CH(p, ':');
+    }
+    ZX_OUT_MEM(p, name, name_len);
+    ZX_OUT_MEM(p, attr->s, attr->len);
+    ZX_OUT_CH(p, '"');
+  }
+  return p;
+}
+
+void zx_dup_attr(struct zx_ctx* c, struct zx_str* attr)
 {
   char* p;
-  for (; attr; attr = (struct zx_str_s*)attr->g.n)
+  for (; attr; attr = (struct zx_str*)attr->g.n)
     if (attr->s) {
       p = ZX_ALLOC(c, attr->len);
       memcpy(p, attr->s, attr->len);
@@ -413,25 +632,25 @@ void zx_dup_attr(struct zx_ctx* c, struct zx_str_s* attr)
     }
 }
 
-void zx_free_attr(struct zx_ctx* c, struct zx_str_s* attr, int free_strs)
+void zx_free_attr(struct zx_ctx* c, struct zx_str* attr, int free_strs)
 {
-  struct zx_str_s* attrn;
+  struct zx_str* attrn;
   for (; attr; attr = attrn) {
-    attrn = (struct zx_str_s*)attr->g.n;
+    attrn = (struct zx_str*)attr->g.n;
     if (free_strs && attr->s)
 	ZX_FREE(c, attr->s);
     ZX_FREE(c, attr);
   }
 }
 
-struct zx_str_s* zx_clone_attr(struct zx_ctx* c, struct zx_str_s* attr)
+struct zx_str* zx_clone_attr(struct zx_ctx* c, struct zx_str* attr)
 {
-  struct zx_str_s* ret;
-  struct zx_str_s* attrnn;
-  struct zx_str_s* attrn;
+  struct zx_str* ret;
+  struct zx_str* attrnn;
+  struct zx_str* attrn;
   char* p;
-  for (attrnn = 0; attr; attr = (struct zx_str_s*)attr->g.n) {
-    ZX_DUPALLOC(c, struct zx_str_s, attrn, attr);
+  for (attrnn = 0; attr; attr = (struct zx_str*)attr->g.n) {
+    ZX_DUPALLOC(c, struct zx_str, attrn, attr);
     if (!attrnn)
       ret = attrn;
     else
@@ -448,10 +667,10 @@ struct zx_str_s* zx_clone_attr(struct zx_ctx* c, struct zx_str_s* attr)
 
 void zx_free_elem_common(struct zx_ctx* c, struct zx_elem_s* x, int free_strs)
 {
-  struct zx_str_s* ss;
+  struct zx_str* ss;
   struct zx_any_attr_s* aa;
   struct zx_any_elem_s* ae;
-  struct zx_str_s* ssn;
+  struct zx_str* ssn;
   struct zx_any_attr_s* aan;
   struct zx_any_elem_s* aen;
   
@@ -473,7 +692,7 @@ void zx_free_elem_common(struct zx_ctx* c, struct zx_elem_s* x, int free_strs)
     zx_FREE_simple_elem(c, &ae->gg, free_strs);
   }
   for (ss = x->content; ss; ss = ssn) {   /* content */
-    ssn = (struct zx_str_s*)ss->g.n;
+    ssn = (struct zx_str*)ss->g.n;
     if (free_strs && ss->s)
       ZX_FREE(c, ss->s);
   }
@@ -482,13 +701,13 @@ void zx_free_elem_common(struct zx_ctx* c, struct zx_elem_s* x, int free_strs)
 
 struct zx_elem_s* zx_clone_elem_common(struct zx_ctx* c, struct zx_elem_s* x, int size, int dup_strs)
 {
-  struct zx_str_s* ss;
+  struct zx_str* ss;
   struct zx_any_attr_s* aa;
   struct zx_any_elem_s* ae;
-  struct zx_str_s* ssn;
+  struct zx_str* ssn;
   struct zx_any_attr_s* aan;
   struct zx_any_elem_s* aen;
-  struct zx_str_s* ssnn;
+  struct zx_str* ssnn;
   struct zx_any_attr_s* aann;
   struct zx_any_elem_s* aenn;
   char* p;
@@ -538,8 +757,8 @@ struct zx_elem_s* zx_clone_elem_common(struct zx_ctx* c, struct zx_elem_s* x, in
     aenn = aen;
   }
   
-  for (ssnn = 0, ss = x->content; ss; ss = (struct zx_str_s*)ss->g.n) {   /* content */
-    ZX_DUPALLOC(c, struct zx_str_s, ssn, ss);
+  for (ssnn = 0, ss = x->content; ss; ss = (struct zx_str*)ss->g.n) {   /* content */
+    ZX_DUPALLOC(c, struct zx_str, ssn, ss);
     if (!ssnn)
       x->content = ssn;
     else
@@ -557,7 +776,7 @@ struct zx_elem_s* zx_clone_elem_common(struct zx_ctx* c, struct zx_elem_s* x, in
 
 void zx_dup_strs_common(struct zx_ctx* c, struct zx_elem_s* x)
 {
-  struct zx_str_s* ss;
+  struct zx_str* ss;
   struct zx_any_attr_s* aa;
   struct zx_any_elem_s* ae;
   char* p;
@@ -587,7 +806,7 @@ void zx_dup_strs_common(struct zx_ctx* c, struct zx_elem_s* x)
   for (ae = x->any_elem; ae; ae = (struct zx_any_elem_s*)ae->gg.g.n)   /* unknown elements */
     zx_DUP_STRS_simple_elem(c, &ae->gg);
 
-  for (ss = x->content; ss; ss = (struct zx_str_s*)ss->g.n)    /* content */
+  for (ss = x->content; ss; ss = (struct zx_str*)ss->g.n)    /* content */
     if (ss->s) {
       p = ZX_ALLOC(c, ss->len);
       memcpy(p, ss->s, ss->len);
@@ -610,7 +829,7 @@ int zx_walk_so_unknown_attributes(struct zx_ctx* c, struct zx_elem_s* x, void* c
 
 int zx_walk_so_unknown_elems_and_content(struct zx_ctx* c, struct zx_elem_s* x, void* ctx, int (*callback)(struct zx_node_s* node, void* ctx))
 {
-  struct zx_str_s* ss;
+  struct zx_str* ss;
   struct zx_any_elem_s* ae;
   int ret;
   
@@ -620,7 +839,7 @@ int zx_walk_so_unknown_elems_and_content(struct zx_ctx* c, struct zx_elem_s* x,
       return ret;
   }
   
-  for (ss = x->content; ss; ss = (struct zx_str_s*)ss->g.n) {   /* content */
+  for (ss = x->content; ss; ss = (struct zx_str*)ss->g.n) {   /* content */
     ret = callback(&ss->g, ctx);
     if (ret)
       return ret;
@@ -679,7 +898,7 @@ int zx_len_simple_elems(struct zx_ctx* c, struct zx_elem_s* se, int siz)
 {
   int len = 0;
   for (; se; se = (struct zx_elem_s*)se->g.n)
-    len += zx_LEN_simple_elem(se, siz);
+    len += zx_LEN_so_simple_elem(se, siz);
   return len;
 }
 
@@ -691,4 +910,13 @@ char* zx_enc_so_simple_elems(struct zx_ctx* c, struct zx_elem_s* se, char* p, ch
 }
 #endif
 
+void zx_prepare_dec_ctx(struct zx_ctx* c, struct zx_ns_s* ns_tab, char* start, char* lim)
+{
+  c->guard_seen_n.seen_n = &c->guard_seen_p;
+  c->guard_seen_p.seen_p = &c->guard_seen_n;
+  c->ns_tab = ns_tab;
+  c->base = c->p = start;
+  c->lim = lim;
+}
+
 /* EOF -- zxlib.c */
